# Ein bischen was zu FUNKTIONEN

# Funktionen sind wichtig für einen strukturierten Programmaufbau. Man kann sie sich als Kurzschreibweise fuer wiederkehrenden Programmcode vorstellen.
# Eine Funktion kann Ein- und Ausgabevariablen haben.
# Eine Funktion hat eine erste und letzte Zeile, die mit "function" bzw mit "endfunction" beginnen. Dazwischen befindet sich die Funktionsdeklaration. 
# Innerhalb der Deklaration koennen Eingabevariablen wie Konstanten verwendet werden, insbesondere kann man sie auch ueberschreiben, wenn man ihren Wert nicht mehr braucht.
# Darueber hinaus sind innerhalb der Funktionsdeklaration keine Variablen von auserhalb der Deklaration verfuegbar! Alle Werte die man braucht muessn also ueber Eingabevariablen uebergeben werden.
# Ist der Code der Funktion abgearbeitet werden die berechneten Werte der Ausgabevariablen an das Programm von dem aus man die Funktion aufgerufen hatte zurueckgegeben. Dies stellt die einzige Moeglichkeit dar, wie Informationen aus einer Funktion herauskommen koennen
# Es ist also moeglich innerhalb einer Funktionsdeklarationen Variablen zu deklarieren, die den Namen einer Variablen ausserhalb der Funktion tragen. Diese haben dann aber nichts miteinander zu tun!

# Syntax:
#
# function [a1, a2, ...] = NAME(e1, e2, ...)
# DEKLARATION
# endfunction
#
# a1,a2 usw bezeichnen Ausgabevariablen. Eine Funktion muss keine haben! In so einem Fall ist die Liste leer und kann weggelassen werden. (man laesst dann natuerlich auch das "=" weg)
# e1,e2 usw sind Eingabevariablen.. Gibt es keine, kann man entweder "NAME()" oder "NAME" schreiben.
# DEKLARATION - hier kommt euer Beitrag

# Funktionen werden mit 
# name(e1,e2,..);
# oder
# [a1 a2 ...] = name(e1,e2,...);


2; # Diese Zeile macht aus dieser Datei ein "Skript". Es waere auch jeder andere Ausdruck moeglich, solange keine Funktion als erstes nach den Kommentaren kommt. Beginnt eine Datei mit einer Funktion, so interpretiert Octave sie als "Funktionsdatei".

# Ein Beispiel ohne Ein- und Ausgabevariablen
function bsp
printf("Diese Funktion hat keine Ein- und Ausgabeargumente\n");
endfunction
# FAZIT: Solche Funktionen machen nur dann Sinn, wenn man viel wiederkehrenden Programmcode hat der von dem uebrigen unabhaengig ist. Also sehr selten.

# Eine standard-Funktion mit einer Eingabe "z" und Ausgabe "erg". Man beachte, dass keine eckigen Klammern um "erg" stehen! (Vergleich: 2 und der [2] bedeuten ja auch da selbe!)
# Die Funktion beschreibt eine Logarithmusfunktion die nur positive Zahlen verarbeitet.
# Was passiert bei negativer Eingabe?
function erg = reallog(z)
if (z>0) 
  erg=log(z);
endif
endfunction

# FAZIT: Immer pruefen, ob die Ausgabevariable fuer jede Eingabe definiert ist! Wenn Man weis, dass etwas schiefgehen kann, sollte man sein Programm dagegen widmen und die Fehler abfangen:

# Diese Log-Funktion ist wie die vorherige, nur dass sie sich bei negativer Eingabe beschwert!
function erg = reallog(z)
  erg=0;
  if(z>0)
    erg=log(z);
  else
    error("Eingabeargument ist negativ");
  endif
endfunction

# FAZIT: "error" beendet die Funktionsauswertung! Deshalb produziert "x=reallog(-1)" zwar eine Fehlermeldung, weist x aber NICHT den Wert 0 zu! Damit ist die Zeile "erg=0;" in der Deklarationueberfluessig!


# Diese Funktion erfüllt zwar keinen Zweck, ist aber lehrreich
function [x y] = sucheIndex(A,c)
    x=y=0;    
    pos=find(A==c);
    x=mod(pos+2,3)+1;
    y=floor((pos-1)/3)+1;
endfunction


# Folgendes Beispiel ist nicht unbedingt noetig zum Erstellen der Zettel. Es gibt euch allerdings noch mehr Flexibilitaet, die mal nuetzlich werden koennte

# Funktionen koennen auch mit weniger Eingabevariablen aufgerufen werden, als eigentlich vorgesehen sind. Das fuehrt nur dann zu einem Fehler, wenn in der Funktionsdeklaration auf die nicht uebergebenen Werte zugegriffen wird. Je nach der Bedeutung der uebergebenen Variablen muss das aber nie passieren.
# Innerhalb einer Funktion kann man die Anzahl der uebergebenen Eingabevariablen mit "nargin()" abfragen und so das Verhalten der Funktion an die Eingabe anpassen.
# Ebenso ist auch ein Funktionsaufruf mit weniger als den in ersten Funktionszeile angegebenen Ausgabevariablen moeglich. Mit dem Befehl "nargout()" kann man innerhalb der Funktionsdeklaration erfragn, wieviel Ausgabewerte erwartet werden.
# Wichtig: Ein- und Ausgabevariablen werden immer von links nach rechts angegeben. Es ist nicht moeglich welche auch der Mitte weg zu lassen.

function [x y]=einUndAusgabe(a,b,c)
  printf("Diese Funktion hat %i Ein- und %i Ausgabevariablen\n",nargin(),nargout());
  x=1;y=2;
endfunction
# Fazit: damit sich keine Fehler einschleichen, oder damit man sie schneller findet, sollte man vielleicht Zeilen folgender Art einbauen
# if nargin()>4
#   error("Zu viele Eingabeargumente")
# endif

# Hier ein paar Demo-Aufrufe. Nur der letzte produziert eine Fehlermeldung und auch nur weil ein Ausgabeargument nicht definiert wird!
# einUndAusgabe
# x= einUndAusgabe(1,2,3)
# [x y] = einUndAusgabe(2)
# einUndAusgabe(1,2,3,4,5)
# [x y z r]= einUndAusgabe(1,2,3,4)


